@@ -1,5 +1,6 @@
-# MP3 stream header information support for Mutagen.
-# Copyright 2006 Joe Wreschnig
+# -*- coding: utf-8 -*-
+
+# Copyright (C) 2006  Joe Wreschnig
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of version 2 of the GNU General Public License as
@@ -10,58 +11,79 @@
 import os
 import struct
 
+from ._compat import endswith
+from mutagen import StreamInfo
+from mutagen._util import MutagenError
 from mutagen.id3 import ID3FileType, BitPaddedInt, delete
 
 __all__ = ["MP3", "Open", "delete", "MP3"]
 
-class error(RuntimeError): pass
-class HeaderNotFoundError(error, IOError): pass
-class InvalidMPEGHeader(error, IOError): pass
+
+class error(RuntimeError, MutagenError):
+    pass
+
+
+class HeaderNotFoundError(error, IOError):
+    pass
+
+
+class InvalidMPEGHeader(error, IOError):
+    pass
+
 
 # Mode values.
 STEREO, JOINTSTEREO, DUALCHANNEL, MONO = range(4)
 
-class MPEGInfo(object):
+
+class MPEGInfo(StreamInfo):
     """MPEG audio stream information
 
     Parse information about an MPEG audio file. This also reads the
     Xing VBR header format.
 
     This code was implemented based on the format documentation at
-    http://www.dv.co.yu/mpgscript/mpeghdr.htm.
+    http://mpgedit.org/mpgedit/mpeg_format/mpeghdr.htm.
 
     Useful attributes:
-    length -- audio length, in seconds
-    bitrate -- audio bitrate, in bits per second
-    sketchy -- if true, the file may not be valid MPEG audio
+
+    * length -- audio length, in seconds
+    * bitrate -- audio bitrate, in bits per second
+    * sketchy -- if true, the file may not be valid MPEG audio
 
     Useless attributes:
-    version -- MPEG version (1, 2, 2.5)
-    layer -- 1, 2, or 3
-    mode -- One of STEREO, JOINTSTEREO, DUALCHANNEL, or MONO (0-3)
-    protected -- whether or not the file is "protected"
-    padding -- whether or not audio frames are padded
-    sample_rate -- audio sample rate, in Hz
+
+    * version -- MPEG version (1, 2, 2.5)
+    * layer -- 1, 2, or 3
+    * mode -- One of STEREO, JOINTSTEREO, DUALCHANNEL, or MONO (0-3)
+    * protected -- whether or not the file is "protected"
+    * padding -- whether or not audio frames are padded
+    * sample_rate -- audio sample rate, in Hz
     """
 
     # Map (version, layer) tuples to bitrates.
     __BITRATE = {
-        (1, 1): range(0, 480, 32),
-        (1, 2): [0, 32, 48, 56, 64, 80, 96, 112,128,160,192,224,256,320,384],
-        (1, 3): [0, 32, 40, 48, 56, 64, 80, 96, 112,128,160,192,224,256,320],
-        (2, 1): [0, 32, 48, 56, 64, 80, 96, 112,128,144,160,176,192,224,256],
-        (2, 2): [0,  8, 16, 24, 32, 40, 48,  56, 64, 80, 96,112,128,144,160],
-        }
-        
+        (1, 1): [0, 32, 64, 96, 128, 160, 192, 224,
+                 256, 288, 320, 352, 384, 416, 448],
+        (1, 2): [0, 32, 48, 56, 64, 80, 96, 112, 128,
+                 160, 192, 224, 256, 320, 384],
+        (1, 3): [0, 32, 40, 48, 56, 64, 80, 96, 112,
+                 128, 160, 192, 224, 256, 320],
+        (2, 1): [0, 32, 48, 56, 64, 80, 96, 112, 128,
+                 144, 160, 176, 192, 224, 256],
+        (2, 2): [0, 8, 16, 24, 32, 40, 48, 56, 64,
+                 80, 96, 112, 128, 144, 160],
+    }
+
     __BITRATE[(2, 3)] = __BITRATE[(2, 2)]
-    for i in range(1, 4): __BITRATE[(2.5, i)] = __BITRATE[(2, i)]
+    for i in range(1, 4):
+        __BITRATE[(2.5, i)] = __BITRATE[(2, i)]
 
     # Map version to sample rates.
     __RATES = {
         1: [44100, 48000, 32000],
         2: [22050, 24000, 16000],
         2.5: [11025, 12000, 8000]
-        }
+    }
 
     sketchy = False
 
@@ -74,7 +96,8 @@ class MPEGInfo(object):
         loading files significantly faster.
         """
 
-        try: size = os.path.getsize(fileobj.name)
+        try:
+            size = os.path.getsize(fileobj.name)
         except (IOError, OSError, AttributeError):
             fileobj.seek(0, 2)
             size = fileobj.tell()
@@ -83,20 +106,26 @@ class MPEGInfo(object):
         if offset is None:
             fileobj.seek(0, 0)
             idata = fileobj.read(10)
-            try: id3, insize = struct.unpack('>3sxxx4s', idata)
-            except struct.error: id3, insize = '', 0
+            try:
+                id3, insize = struct.unpack('>3sxxx4s', idata)
+            except struct.error:
+                id3, insize = '', 0
             insize = BitPaddedInt(insize)
-            if id3 == 'ID3' and insize > 0:
-                offset = insize
-            else: offset = 0
+            if id3 == b'ID3' and insize > 0:
+                offset = insize + 10
+            else:
+                offset = 0
 
         # Try to find two valid headers (meaning, very likely MPEG data)
         # at the given offset, 30% through the file, 60% through the file,
         # and 90% through the file.
         for i in [offset, 0.3 * size, 0.6 * size, 0.9 * size]:
-            try: self.__try(fileobj, int(i), size - offset)
-            except error, e: pass
-            else: break
+            try:
+                self.__try(fileobj, int(i), size - offset)
+            except error:
+                pass
+            else:
+                break
         # If we can't find any two consecutive frames, try to find just
         # one frame back at the original offset given.
         else:
@@ -114,11 +143,11 @@ class MPEGInfo(object):
         # is assuming the offset didn't lie.
         data = fileobj.read(32768)
 
-        frame_1 = data.find("\xff")
-        while 0 <= frame_1 <= len(data) - 4:
+        frame_1 = data.find(b"\xff")
+        while 0 <= frame_1 <= (len(data) - 4):
             frame_data = struct.unpack(">I", data[frame_1:frame_1 + 4])[0]
-            if (frame_data >> 16) & 0xE0 != 0xE0:
-                frame_1 = data.find("\xff", frame_1 + 2)
+            if ((frame_data >> 16) & 0xE0) != 0xE0:
+                frame_1 = data.find(b"\xff", frame_1 + 2)
             else:
                 version = (frame_data >> 19) & 0x3
                 layer = (frame_data >> 17) & 0x3
@@ -126,16 +155,17 @@ class MPEGInfo(object):
                 bitrate = (frame_data >> 12) & 0xF
                 sample_rate = (frame_data >> 10) & 0x3
                 padding = (frame_data >> 9) & 0x1
-                private = (frame_data >> 8) & 0x1
+                # private = (frame_data >> 8) & 0x1
                 self.mode = (frame_data >> 6) & 0x3
-                mode_extension = (frame_data >> 4) & 0x3
-                copyright = (frame_data >> 3) & 0x1
-                original = (frame_data >> 2) & 0x1
-                emphasis = (frame_data >> 0) & 0x3
+                # mode_extension = (frame_data >> 4) & 0x3
+                # copyright = (frame_data >> 3) & 0x1
+                # original = (frame_data >> 2) & 0x1
+                # emphasis = (frame_data >> 0) & 0x3
                 if (version == 1 or layer == 0 or sample_rate == 0x3 or
-                    bitrate == 0 or bitrate == 0xF):
-                    frame_1 = data.find("\xff", frame_1 + 2)
-                else: break
+                        bitrate == 0 or bitrate == 0xF):
+                    frame_1 = data.find(b"\xff", frame_1 + 2)
+                else:
+                    break
         else:
             raise HeaderNotFoundError("can't sync to an MPEG frame")
 
@@ -151,39 +181,43 @@ class MPEGInfo(object):
         self.sample_rate = self.__RATES[self.version][sample_rate]
 
         if self.layer == 1:
-            frame_length = (12 * self.bitrate / self.sample_rate + padding) * 4
+            frame_length = (
+                (12 * self.bitrate // self.sample_rate) + padding) * 4
             frame_size = 384
         elif self.version >= 2 and self.layer == 3:
-            frame_length = 72 * self.bitrate / self.sample_rate + padding
+            frame_length = (72 * self.bitrate // self.sample_rate) + padding
             frame_size = 576
         else:
-            frame_length = 144 * self.bitrate / self.sample_rate + padding
+            frame_length = (144 * self.bitrate // self.sample_rate) + padding
             frame_size = 1152
 
         if check_second:
-            possible = frame_1 + frame_length
+            possible = int(frame_1 + frame_length)
             if possible > len(data) + 4:
                 raise HeaderNotFoundError("can't sync to second MPEG frame")
-            frame_data = struct.unpack(">H", data[possible:possible + 2])[0]
-            if frame_data & 0xFFE0 != 0xFFE0:
+            try:
+                frame_data = struct.unpack(
+                    ">H", data[possible:possible + 2])[0]
+            except struct.error:
+                raise HeaderNotFoundError("can't sync to second MPEG frame")
+            if (frame_data & 0xFFE0) != 0xFFE0:
                 raise HeaderNotFoundError("can't sync to second MPEG frame")
 
-        frame_count = real_size / float(frame_length)
-        samples = frame_size * frame_count
-        self.length = samples / self.sample_rate
+        self.length = 8 * real_size / float(self.bitrate)
 
         # Try to find/parse the Xing header, which trumps the above length
         # and bitrate calculation.
         fileobj.seek(offset, 0)
         data = fileobj.read(32768)
         try:
-            xing = data[:-4].index("Xing")
+            xing = data[:-4].index(b"Xing")
         except ValueError:
             # Try to find/parse the VBRI header, which trumps the above length
             # calculation.
             try:
-                vbri = data[:-24].index("VBRI")
-            except ValueError: pass
+                vbri = data[:-24].index(b"VBRI")
+            except ValueError:
+                pass
             else:
                 # If a VBRI header was found, this is definitely MPEG audio.
                 self.sketchy = False
@@ -202,42 +236,54 @@ class MPEGInfo(object):
                 samples = float(frame_size * frame_count)
                 self.length = (samples / self.sample_rate) or self.length
             if flags & 0x2:
-                bytes = struct.unpack('>I', data[xing + 12:xing + 16])[0]
-                self.bitrate = int((bytes * 8) // self.length)
-
-        # If the bitrate * the length is nowhere near the file
-        # length, recalculate using the bitrate and file length.
-        # Don't do this for very small files.
-        fileobj.seek(2, 0)
-        size = fileobj.tell()
-        expected = (self.bitrate / 8) * self.length
-        if not (size / 2 < expected < size * 2) and size > 2**16:
-            self.length = size / float(self.bitrate * 8)
+                bitrate_data = struct.unpack(
+                    '>I', data[xing + 12:xing + 16])[0]
+                self.bitrate = int((bitrate_data * 8) // self.length)
 
     def pprint(self):
         s = "MPEG %s layer %d, %d bps, %s Hz, %.2f seconds" % (
             self.version, self.layer, self.bitrate, self.sample_rate,
             self.length)
-        if self.sketchy: s += " (sketchy)"
+        if self.sketchy:
+            s += " (sketchy)"
         return s
 
+
 class MP3(ID3FileType):
-    """An MPEG audio (usually MPEG-1 Layer 3) file."""
+    """An MPEG audio (usually MPEG-1 Layer 3) file.
+
+    :ivar info: :class:`MPEGInfo`
+    :ivar tags: :class:`ID3 <mutagen.id3.ID3>`
+    """
 
     _Info = MPEGInfo
-    _mimes = ["audio/mp3", "audio/x-mp3", "audio/mpeg", "audio/mpg",
-              "audio/x-mpeg"]
 
-    def score(filename, fileobj, header):
+    _mimes = ["audio/mpeg", "audio/mpg", "audio/x-mpeg"]
+
+    @property
+    def mime(self):
+        l = self.info.layer
+        return ["audio/mp%d" % l, "audio/x-mp%d" % l] + super(MP3, self).mime
+
+    @staticmethod
+    def score(filename, fileobj, header_data):
         filename = filename.lower()
-        return (header.startswith("ID3") * 2 + filename.endswith(".mp3") +
-                filename.endswith(".mp2") + filename.endswith(".mpg") +
-                filename.endswith(".mpeg"))
-    score = staticmethod(score)
+
+        return (header_data.startswith(b"ID3") * 2 +
+                endswith(filename, b".mp3") +
+                endswith(filename, b".mp2") + endswith(filename, b".mpg") +
+                endswith(filename, b".mpeg"))
+
 
 Open = MP3
 
-class EasyMP3(MP3):
-    """Like MP3, but uses EasyID3 for tags."""
-    from mutagen.easyid3 import EasyID3 as ID3
 
+class EasyMP3(MP3):
+    """Like MP3, but uses EasyID3 for tags.
+
+    :ivar info: :class:`MPEGInfo`
+    :ivar tags: :class:`EasyID3 <mutagen.easyid3.EasyID3>`
+    """
+
+    from mutagen.easyid3 import EasyID3 as ID3
+    ID3 = ID3
